drop schema if exists mydb;

create schema mydb;

use mydb;

ALTER DATABASE mydb DEFAULT CHARACTER SET utf8;

CREATE TABLE Location (
    location_id int NOT NULL auto_increment,
    latitude DOUBLE NOT NULL,
    longitude DOUBLE NOT NULL,
	name VARCHAR(150) NOT NULL,
	formatted_address VARCHAR(250) NOT NULL,
	secondary_id VARCHAR(100) NOT NULL,
    PRIMARY KEY (location_id)
);


INSERT INTO Location VALUES(DEFAULT, 39.908859, 32.778438, "Cepa Ankara", "Eskişehir Yolu 7.km 06520, 06520 Ankara, Turkey" , "seconday_id");


CREATE TABLE User (
    user_id int NOT NULL auto_increment,
    email VARCHAR(50) NOT NULL,
    password VARCHAR(30) NOT NULL,
    first_name VARCHAR(50) NOT NULL,
    last_name VARCHAR(30) NOT NULL,
    gender VARCHAR(10) NOT NULL,
    date_of_birth DATE NOT NULL,
	city varchar(50) NOT NULL,
    PRIMARY KEY (user_id),
    UNIQUE (email)
);

INSERT INTO User VALUES (DEFAULT, 'john@doe.com', '12345', 'john', 'doe', 'male', '2000-01-01', 'Istanbul');
INSERT INTO User VALUES (DEFAULT, 'e1746429@ceng.metu.edu.tr', '98765', 'yasar baris', 'ulu', 'female', '1991-01-27', 'Ankara');





CREATE TABLE Friendship (
    user_id int NOT NULL,
    friend_user_id int NOT NULL,
    PRIMARY KEY (user_id , friend_user_id)
);


INSERT INTO Friendship VALUES ( 1,2 ) , (2,1);


CREATE TABLE Badge (
    badge_id int NOT NULL,
    badge_name VARCHAR(30) NOT NULL,
    badge_info VARCHAR(200) NOT NULL,
    PRIMARY KEY (badge_id)
);



INSERT INTO Badge VALUES ( 1 ,  "creator" , "this badge is given to only special ones...");


CREATE TABLE Level (
    user_id int NOT NULL,
    level int NOT NULL,
    experiece_for_next int NOT NULL,
    PRIMARY KEY (user_id)
);



INSERT INTO Level VALUES ( 1 , 26 , 500);


CREATE TABLE Event (
    event_id int NOT NULL AUTO_INCREMENT,
    type VARCHAR(50),
    name VARCHAR(50) NOT NULL,
    description VARCHAR(250),
    location_id int NOT NULL,
    start_time DATE NOT NULL,
    end_time DATE NOT NULL,
	isPrivate int NOT NULL,
    PRIMARY KEY (event_id),
    FOREIGN KEY (location_id)
        REFERENCES Location (location_id)
);


CREATE TABLE `EventAccess` (
  `event_id` int(11) NOT NULL DEFAULT '0',
  `user_id` int(11) NOT NULL DEFAULT '0',
  PRIMARY KEY (`event_id`,`user_id`),
  KEY `user_id` (`user_id`),
  CONSTRAINT `EventAccess_ibfk_1` FOREIGN KEY (`event_id`) REFERENCES `Event` (`event_id`),
  CONSTRAINT `EventAccess_ibfk_2` FOREIGN KEY (`user_id`) REFERENCES `User` (`user_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;



INSERT INTO Event VALUES(DEFAULT, "party", "happy new year", "we will celebrate new year",1, "2013-12-31 11:00:00 PM", "2014-01-01 04:00:00 AM",0);

INSERT INTO Event VALUES(DEFAULT, "sport", "FB - GS", "leage 11th week", 1 , "2013-11-10 7:00:00 PM", "2013-11-10 7:00:00 PM",0);



CREATE TABLE CreateEvent (
    user_id int NOT NULL,
    event_id int NOT NULL,
    PRIMARY KEY (user_id , event_id),
    FOREIGN KEY (user_id)
        REFERENCES User (user_id),
    FOREIGN KEY (event_id)
        REFERENCES Event (event_id)
);



INSERT INTO CreateEvent VALUES (1,2);




CREATE TABLE JoinEvent (
    event_id int NOT NULL,
    user_id int NOT NULL,
    PRIMARY KEY (user_id , event_id),
    FOREIGN KEY (user_id)
        REFERENCES User (user_id),
    FOREIGN KEY (event_id)
        REFERENCES Event (event_id)
);



INSERT INTO JoinEvent VALUES (1,1);
INSERT INTO JoinEvent VALUES (2,1);



CREATE TABLE ShareEvent (
    user_id int NOT NULL,
    event_id int NOT NULL,
    PRIMARY KEY (user_id , event_id),
    FOREIGN KEY (user_id)
        REFERENCES User (user_id),
    FOREIGN KEY (event_id)
        REFERENCES Event (event_id)
);


INSERT INTO ShareEvent VALUES (1,2);



CREATE TABLE PromoteEvent (
    user_id int NOT NULL,
    event_id int NOT NULL,
    PRIMARY KEY (user_id , event_id),
    FOREIGN KEY (user_id)
        REFERENCES User (user_id),
    FOREIGN KEY (event_id)
        REFERENCES Event (event_id)
);

INSERT INTO PromoteEvent VALUES (1,2);




CREATE TABLE RateEvent (
    event_id int NOT NULL,
    user_id int NOT NULL,
    point int NOT NULL,
    PRIMARY KEY (user_id , event_id),
    FOREIGN KEY (user_id)
        REFERENCES User (user_id),
    FOREIGN KEY (event_id)
        REFERENCES Event (event_id)
);

INSERT INTO RateEvent VALUES (2,1,5);
INSERT INTO RateEvent VALUES (1,1,8);

CREATE TABLE CommentOnEvent (
    event_id int NOT NULL,
    user_id int NOT NULL,
    comment VARCHAR(500) NOT NULL,
    PRIMARY KEY (user_id , event_id),
    FOREIGN KEY (user_id)
        REFERENCES User (user_id),
    FOREIGN KEY (event_id)
        REFERENCES Event (event_id)
);

INSERT INTO CommentOnEvent VALUES (2,1,"terrible");
INSERT INTO CommentOnEvent VALUES (1,1,"perfect");





CREATE TABLE Follow (
    follower_user_id int NOT NULL,
    user_id int NOT NULL,
    PRIMARY KEY (follower_user_id , user_id)
);


INSERT INTO Follow VALUES ( 1,2 ) ;

CREATE TABLE FriendRequest (
    user_id int NOT NULL,
    requested_user_id int NOT NULL,
    PRIMARY KEY (user_id , requested_user_id)
);

CREATE TABLE `Newsfeed` (
  `user_id` int(11) NOT NULL DEFAULT '0',
  `event_id` int(11) NOT NULL DEFAULT '0',
  `operation` varchar(50) NOT NULL DEFAULT '',
  `server_time` date DEFAULT NULL,
  PRIMARY KEY (`user_id`,`event_id`,`operation`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


INSERT INTO FriendRequest VALUES ( 1,2 );

ALTER TABLE Badge CONVERT TO CHARACTER SET utf8;
ALTER TABLE CommentOnEvent CONVERT TO CHARACTER SET utf8;
ALTER TABLE CreateEvent CONVERT TO CHARACTER SET utf8;
ALTER TABLE Event CONVERT TO CHARACTER SET utf8;
ALTER TABLE Follow CONVERT TO CHARACTER SET utf8;
ALTER TABLE FriendRequest CONVERT TO CHARACTER SET utf8;
ALTER TABLE Friendship CONVERT TO CHARACTER SET utf8;
ALTER TABLE JoinEvent CONVERT TO CHARACTER SET utf8;
ALTER TABLE Level CONVERT TO CHARACTER SET utf8;
ALTER TABLE PromoteEvent CONVERT TO CHARACTER SET utf8;
ALTER TABLE RateEvent CONVERT TO CHARACTER SET utf8;
ALTER TABLE ShareEvent CONVERT TO CHARACTER SET utf8;
ALTER TABLE User CONVERT TO CHARACTER SET utf8;
ALTER TABLE Location CONVERT TO CHARACTER SET utf8;
