package tr.cerealkillers.com.Controller;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import org.primefaces.event.map.OverlaySelectEvent;
import org.primefaces.json.JSONArray;
import org.primefaces.json.JSONException;
import org.primefaces.json.JSONObject;
import org.primefaces.model.map.DefaultMapModel;
import org.primefaces.model.map.LatLng;
import org.primefaces.model.map.MapModel;
import org.primefaces.model.map.Marker;
import tr.cerealkillers.com.Objects.EventImplementation;
import tr.cerealkillers.com.Objects.GoogleLocation;
import tr.cerealkillers.com.Util.WebServiceConnector;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@ManagedBean
@SessionScoped
public class EventController implements Serializable {
	@ManagedProperty(value = "#{loginController}")
	private LoginController loginController;

	public void setLoginController(LoginController loginController) {
		this.loginController = loginController;
	}
	
	private List<EventImplementation> events;
	private int eventid;
	private String type;
	private String name;
	private String description;
	private int locationid = 1;
	private double latitude;
	private double longitude;
	private String address;
	private Date startTime;
	private Date endTime;
	private String message;
	private String city;
	private String country = "Türkiye";
	private int stepCount = 1;
	private MapModel map;
	private Marker marker;
	private List<GoogleLocation> results;
	private double firstLat;
	private double firstLng;
	private int isPrivate;
	
	
	public int getIsPrivate() {
		return isPrivate;
	}

	public void setIsPrivate(int isPrivate) {
		this.isPrivate = isPrivate;
	}

	public EventController() {
		map = new DefaultMapModel();
	}

	public double getFirstLat() {
		return firstLat;
	}

	public void setFirstLat(double firstLat) {
		this.firstLat = firstLat;
	}

	public double getFirstLng() {
		return firstLng;
	}

	public void setFirstLng(double firstLng) {
		this.firstLng = firstLng;
	}

	public Marker getMarker() {
		return marker;
	}

	public void setMarker(Marker marker) {
		this.marker = marker;
	}

	@ManagedProperty("#{mapBean}")
	private MapBean mapBean;

	public MapBean getMapBean() {
		return mapBean;
	}

	public void setMapBean(MapBean mapBean) {
		this.mapBean = mapBean;
	}

	public MapModel getMap() {
		return map;
	}

	public void setMap(MapModel map) {
		this.map = map;
	}

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public int getStepCount() {
		return stepCount;
	}

	public void setStepCount(int stepCount) {
		this.stepCount = stepCount;
	}

	public List<EventImplementation> getEvents() {
		return events;
	}

	public void setEvents(List<EventImplementation> events) {
		this.events = events;
	}

	public int getEventid() {
		return eventid;
	}

	public void setEventid(int eventid) {
		this.eventid = eventid;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public int getLocationid() {
		return locationid;
	}

	public void setLocationid(int locationid) {
		this.locationid = locationid;
	}

	public double getLatitude() {
		return latitude;
	}

	public void setLatitude(double latitude) {
		this.latitude = latitude;
	}

	public double getLongitude() {
		return longitude;
	}

	public void setLongitude(double longitude) {
		this.longitude = longitude;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

	public List<GoogleLocation> getResults() {
		return results;
	}

	public void setResults(List<GoogleLocation> results) {
		this.results = results;
	}

	public void onMarkerSelect(OverlaySelectEvent event) {
		marker = (Marker) event.getOverlay();
		FacesContext.getCurrentInstance().addMessage(null,
				new FacesMessage("Address selected: " + marker.getTitle()));
	}

	private void init() {
		map = new DefaultMapModel();
		results = new ArrayList<GoogleLocation>();
		type = "";
		name = "";
		description = "";
		startTime = null;
		endTime = null;
		city = "";
		address = "";
		marker = null;
	}

	public void findLocation() {
		String result = WebServiceConnector.getInstance().connectGet(
				"rest/location/searchLocation/" + address);
		try {
			JSONObject jsonAddress = new JSONObject(result);

			if (jsonAddress.getString("status").equals("OK")) {
				map = new DefaultMapModel();
				results = new ArrayList<GoogleLocation>();
				JSONArray jarray = jsonAddress.getJSONArray("results");
				for (int i = 0; i < jarray.length(); i++) {
					JSONObject details = jarray.getJSONObject(i);
					String addr = details.getString("formatted_address");
					String name = details.getString("name");
					String googleId = "";
					double lat = 0;
					double lng = 0;
                    int id = 0;
					if (details.has("geometry")) {
						JSONObject location = details.getJSONObject("geometry")
								.getJSONObject("location");
						lat = location.getDouble("lat");
						lng = location.getDouble("lng");
						googleId = details.getString("id");
					} else {
						googleId = details.getString("secondary_id");
                        id = details.getInt("location_id");
						lat = details.getDouble("latitude");
						lng = details.getDouble("longitude");
					}
					results.add(new GoogleLocation(id,name, addr, lat, lng,
							googleId));
                    Marker marker = new Marker(new LatLng(lat, lng), addr);
                    marker.setIcon("/marker.png");
					map.addOverlay(marker);
					if (i == 0) {
						firstLat = lat;
						firstLng = lng;
					}

				}
				return;
			} else {
				message = "Cannot find the address: " + address;
				FacesContext context = FacesContext.getCurrentInstance();
				context.addMessage(null, new FacesMessage(message));
				return;
			}

		} catch (JSONException e) {
			e.printStackTrace();
		}
		FacesContext context = FacesContext.getCurrentInstance();
		context.addMessage(null, new FacesMessage(
				"An error occured that we couldn't figure out why"));
	}

	public String createEvent() {
		SimpleDateFormat formatter = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
		JSONObject jsonObject = new JSONObject();
		if (marker == null) {
			FacesContext context = FacesContext.getCurrentInstance();
			context.addMessage(null, new FacesMessage(
					"Please select a marker before creating the event"));
			return "event";
		}
		try {
			int index = map.getMarkers().indexOf(marker);
			jsonObject.put("user_id", loginController.getTheUser().getUserid());
			jsonObject.put("event_type", type);
			jsonObject.put("event_name", name);
			jsonObject.put("event_description", description);
			jsonObject.put("latitude", results.get(index).getLat());
			jsonObject.put("longitude", results.get(index).getLon());
			jsonObject.put("start_time", formatter.format(startTime));
			jsonObject.put("end_time", formatter.format(endTime));
			jsonObject.put("formatted_address", results.get(index)
					.getFormattedAddress());
			jsonObject.put("name", results.get(index).getName());
			jsonObject.put("secondary_id", results.get(index).getGoogleId());
			jsonObject.put("isPrivate", isPrivate);
			String response = WebServiceConnector.getInstance().connectPost(
					"/rest/event/createEvent/", jsonObject);
			JSONObject json = new JSONObject(response);
			response = json.getString("message");
			if (response.equals("Event Created")) {
				message = "Event created succesfully";
			} else {
				message = "Event creation failed";
			}
		} catch (JSONException e) {
		}
		FacesContext context = FacesContext.getCurrentInstance();
		context.addMessage(null, new FacesMessage(message));
		if (message.endsWith("failed")) {
			return "event";
		}
		init();
		mapBean.getNearbyEvents(loginController.getTheUser().getUserid());
		return "welcome";

	}

	public String setUserEvents() {
		String response = WebServiceConnector.getInstance().connectGet(
				"/rest/event/getEventsOfUser/"
						+ loginController.getTheUser().getUserid());
		Gson gson = new GsonBuilder().setDateFormat("dd-MM-yyyy HH:mm:ss").create();
		events = gson.fromJson(response,
				new TypeToken<List<EventImplementation>>() {
				}.getType());
		return "profile";
	}

}
